# coding: utf-8
require 'mime/types'

class Dav < DAV4Rack::Resource

  #unloadable

  def initialize(public_path, path, request, response, options)
    _force_encoding!(public_path)
    _force_encoding!(path)
    super(public_path, path, request, response, options)
  end

  def child elemento
    basepath = file_path=="/" ? "/" : file_path + "/"
    path = basepath + (elemento.class.name == "Documento" ? elemento.adjunto_file_name : elemento[:nombre])
    public_path = @options[:root_uri_path] + path
    self.class.new(public_path, path, @request, @response, @options)
  end

  # If this is a collection, return the child resources.
  def children
    #puts "--------------> Nos piden los hijos de " + file_path
    contenido = Espacio.contenido(@authed, stat.id)
    return contenido[:espacios].collect{|o| child o} + contenido[:documentos].collect{|o| child o}
  end

  # Is this resource a collection?
  def collection?
    #puts "-----------> Nos preguntan si " + file_path + " / " + path + " es un directorio"
    #puts "             y no lo es" unless existe_espacio?(file_path)
    existe_espacio?(file_path)
  end

  # Does this recource exist?
  def exist?
    #puts "------------------> " + file_path + " EXISTE!!!" if existe_recurso?(file_path)
    #puts "------------------> " + file_path + " NO EXISTE!!!" unless existe_recurso?(file_path)
    existe_recurso?(file_path)
  end

  # Return the creation time.
  def creation_date
    stat.created_at || Date.new
  end

  # Return the time of last modification.
  def last_modified
    stat.updated_at || Date.new
  end

  # Set the time of last modification.
  def last_modified=(time)
    #File.utime(Time.now, time, file_path)
  end

  # Return an Etag, an unique hash value for this resource.
  def etag
    sprintf('%s-%x-%x', stat.class.name, stat.id||0, stat.updated_at||Time.now)
  end

  # Return the mime type of this resource.
  def content_type
    stat.directory? ? "text/html" : stat.adjunto_content_type
  end

  # Return the size in bytes for this resource.
  def content_length
    stat.class.name == "Documento" ? stat.adjunto_file_size : 0
  end

  #
  # Metodos HTTP permitidos
  #

  # HTTP GET request.
  #
  # Write the content of the resource to the response.body.
  def get(request, response)
    #puts "----------------------> Paseamos por el get"
    raise NotFound unless exist?
    if stat.directory?
      response.body = "<html>"
      response.body << "<h2>" + file_path + "</h2>"
      unless espacio_raiz?(file_path)
        response.body << "<a href='" + File.join(@options[:root_uri_path], ruta_padre(file_path)) + "'>..</a>"
        response.body << "</br>"
        ruta_base = stat.nombre
      else
        ruta_base = "." 
      end
      children.each do |child|
        response.body << "<a href='" + child.public_path.html_safe + "'>" + child.file_name.html_safe + "</a>"
        response.body << "</br>"
      end
      response.body << "</br>"
      response.body << "</html>"
      response['Content-Length'] = response.body.bytesize.to_s
      response['Content-Type'] = 'text/html; charset=UTF-8'
    else
      if File.exists?(stat.adjunto.path)
        response['Content-Type'] = stat.adjunto_content_type
        response.body = File.open(stat.adjunto.path, 'r')
      else
        raise NotFound
      end
    end
    OK
  end

  # HTTP PUT request.
  #
  # Save the content of the request.body.
  def put(request, response)
    espacio_padre = espacio ruta_padre
    #puts "---------------> ESTAMOS EN UN PUT!!!!"
    #puts "                 Enviando: " + file_path 
    #puts "                 Al espacio (" + ruta_padre + "): " + espacio_padre.inspect
    # Si existe el espacio padre y se puede escribir en el
    if espacio_padre && espacio_padre.escritura_permitida(@authed)
      # Si se puede escribir en el fichero o no existe
      if stat.id.nil? || stat.escritura_permitida(@authed)
        nuevo = true
        # Si el fichero existe y no es un directorio...
        if stat.id && !stat.directory?
          # Si es un original, lo sustituye (para no perder los enlaces)
          if stat.espacio.first == espacio_padre
            documento = stat
            nuevo = false
          # y si es un enlace, decimos que no podemos pisarlo...
          else
            raise Forbidden
          end
        else
          documento = Documento.new(:descripcion => _("Subido vía webdav"), :usuario_id => @authed.id)
        end
        # Obtenemos el fichero subido
        uploaded_file = extract_tempfile(request)
        # y lo guardamos
        documento.adjunto = uploaded_file
        documento.save
        if documento.errors.empty?
          # Vincula el documento al espacio padre si es nuevo
          espacio_padre.documento << documento if nuevo 
          Created
        end
      else
        #puts "----------------> No se puede escribir en el fichero"
        raise Forbidden
      end
    else
      #puts "----------------> No se puede escribir:"
      #puts "                  El directorio destino no existe" unless espacio_padre
      #puts "                  No tenemos permisos para escribir sobre el directorio" unless espacio_padre && espacio_padre.escritura_permitida(@authed)
      raise Forbidden 
    end
  end

  # HTTP POST request.
  #
  # Usually forbidden.
  def post(request, response)
    raise MethodNotAllowed
  end

  # HTTP DELETE request.
  #
  # Delete this resource.
  def delete
    espacio_padre = espacio ruta_padre
    # Comprobamos que el usuario pueda borrar el fichero y escribir sobre el directorio
    if espacio_padre && !(stat.espacio.size > 1 && stat.espacio.first == espacio_padre)
      # Si el documento no es unico y es el original, no deja borrarlo
      borra_documento stat, espacio_padre
    else
      #puts "---------------> No se puede borrar: No existe el espacio padre" unless espacio_padre
      #puts "                 El documento esta vinculado en otro espacio" if stat.espacio.size > 1 && stat.espacio.first == espacio_padre
      raise Forbidden
    end
  end

  # HTTP COPY request.
  #
  # Copy this resource to given destination resource.
  def copy(dest, overwrite = false)
    #puts "---------------> Estamos en el COPY"
    #puts "                 Nos dicen que copiemos " + file_path + " sobre " + dest.file_path
    if dest.path == path
      #puts "---------------> ERROR: El origen y el destino es el mismo"
      Conflict
    # El recurso origen y el destino es el mismo 
    elsif dest.stat == stat
      Created
    # No se pueden copiar directorios
    elsif stat.directory?
      #puts "---------------> ERROR: El origen es un directorio o no existe"
      Forbidden
    # Si el destino ya existe y no hemos dicho que se pueda sobreescribir falla
    elsif !dest.stat.directory? && dest.stat.id && !overwrite
      puts "---------------> ERROR: No podemos sobreescribir el fichero destino"
      PreconditionFailed
    # Si el destino existe y se puede sobreescribir pero no tenemos permisos, falla
    elsif overwrite && dest.stat.id && !dest.stat.escritura_permitida(@authed)
      #puts "---------------> ERROR: No tenemos permisos para sobreescribir el fichero destino"
      Forbidden
    else
      # Si el destino es un directorio
      if dest.stat.directory? && dest.stat.id
        ruta_destino = dest.file_path
        nuevo = true 
      # Si es un fichero, trabajamos sobre el padre
      else
        #puts "---------------> Lo copiamos sobre: " + espacio(ruta_padre(dest.file_path)).inspect
        ruta_destino = ruta_padre(dest.file_path) 
        nuevo = dest.stat.id.nil?
      end
      # Comprobamos que exista el espacio destino y que sea escribible
      espacio_destino = espacio ruta_destino
      if espacio_destino && espacio_destino.escritura_permitida(@authed)
        # Si es nuevo lo crea tal cual
        if nuevo
          espacio_destino.documento << stat
          Created
        # Si no es nuevo
        else
          # en el caso de que solo haya uno o estemos modificando el original primero borramos
          if dest.stat.espacio.size == 1 || dest.stat.espacio.first != espacio_destino
            #puts "-------------> Existe y es unico o es un enlace... borrando de " + dest.path
            result = borra_documento dest.stat, espacio_destino
            if result == NoContent
              espacio_destino.documento << stat
              Created
            end
          # si es el original de un enlace tenemos que cambiar todos los enlaces
          # FALTA
          else
            #puts "-------------> es el original de un enlace"
            Forbidden 
          end
        end
      else
        #puts "---------------> ERROR: No tenemos permisos para escribir en el espacio destino " + dest.file_path + "): " + dest.stat.inspect
        Forbidden
      end 
    end
  end

  # Move this resource to given destination resource.
  def move(dest, overwrite = false)
    espacio_padre = espacio ruta_padre
    espacio_destino = espacio ruta_padre(dest.file_path)
    # Si tratamos de mover un directorio o no existe el recurso, error
    if stat.directory? || stat.id.nil?
      #puts "---------------> ERROR: No se pueden mover directorios"
      Forbidden
    # Si no tenemos permisos de escritura sobre el documento o su espacio padre, error
    elsif !stat.escritura_permitida(@authed) || espacio_padre.nil? || !espacio_padre.escritura_permitida(@authed)
      #puts "---------------> ERROR: No se puede escribir en el fichero o en el directorio padre"
      Forbidden
    elsif !espacio_destino.id || !espacio_destino.escritura_permitida(@authed)
      #puts "---------------> ERROR: No se puede escribir en el espacio destino"
      Forbidden
    # Si el recurso ya existe y es un documento lo copiamos
    elsif dest.id && !dest.stat.directory?
      # Primero lo copiamos
      result = copy(dest, overwrite)
      # Si fue bien, borramos el antiguo
      borra_documento(stat, espacio_padre) if [Created, NoContent].include?(result)
      result 
    # Movemos el recurso
    elsif dest.id.nil? 
      dxe = espacio_padre.documento_x_espacio.find_by_documento_id(stat.id) 
      dxe.espacio_id = espacio_destino.id
      dxe.save
      Created
    else 
      Forbidden
    end
  end

  # HTTP MKCOL request.
  #
  # Create this resource as collection.
  def make_collection
    # Si el espacio ya existe lo dejamos pasar y si no damos error (no se pueden crear directorios)
    if stat.directory? && stat.id
      Created
    else
      Forbidden
    end
  end

protected

  def root
    @options[:root]
  end

  def file_path
    File.join(root,path)
  end

  def file_name
    path.split("/")[-1]
  end

  # Store object
  def stat
    unless @stat
      @stat = existe_espacio?(file_path) ? espacio(file_path) : documento(file_path)
    end
    return @stat || Espacio.new
  end


private

  def authenticate(username, password)
    #puts "------------------> Estamos autentificando!!!"
    @authed ||= Usuario.identificacion(username, password) if username && password
    !@authed.nil?
  end

  def _force_encoding!(str)
    str.force_encoding('UTF-8')
  end

  def espacio_id path
    objeto(path, true)
  end

  def recurso_id path
    objeto(path, false)
  end

  # Elimina un documento de un espacio
  def borra_documento documento=stat, espacio=nil
    #puts "-------------------> Nos piden borrar el documento: " + documento.inspect
    return Forbidden unless documento.class.name == "Documento"
    espacio ||= documento.espacio.first
    # Solo borramos si podemos escribir en el espacio, tenemos permisos de escritura en el documento
    if espacio.escritura_permitida(@authed) && stat.escritura_permitida(@authed, espacio)
      espacio.documento.delete(documento)
      # Si el documento no esta vinculado a ningun otro objeto lo borramos
      documento.destroy if documento.espacio.reload.empty?
      #puts "---------------> Documento borrado!"
      NoContent
    else
      #puts "---------------> No se puede borrar:"
      #puts "                 No tenemos permitida la escritura en el espacio padre" unless espacio && espacio.escritura_permitida(@authed)
      #puts "                 No tenemos permitida la escritura sobre el fichero" unless documento.escritura_permitida(@authed, espacio)
      Forbidden
    end
  end

  # Save and return the tempfile. This is slightly duplicative of how Rack/Rails save a tempfile, then Paperclip automatically
  # handles copying it. In our case, since WebDAV isn't like a form multipart upload, we have to explicitly save it
  # rather than having Rails implicitly handle this for us.
  def extract_tempfile(request)
    filename = file_path.split("/")[-1]
    contenttype = MIME::Types.type_for(filename).first
    if request.body.class.name == 'PhusionPassenger::Utils::RewindableInput'
      #puts "------> Estamos desde Passenger"
      uploaded_file = Paperclip::StringioAdapter.new(request.body)
    else
      #puts "------> No estamos en Passenger"
      uploaded_file = Paperclip.io_adapters.for(request.body)
      #uploaded_file.rewind
    end
    uploaded_file.original_filename = filename 
    uploaded_file.content_type = contenttype == "" ? 'application/octet-stream' : contenttype
    #puts "-------------> Content-type: " + uploaded_file.content_type
    return uploaded_file
  end

  def objeto path, solo_espacios=false
    existe = true
    objeto_id = nil
    # Secuencia completa del path
    secuencia = path.split('/')
    # Si el ultimo es una barra, la eliminamos
    secuencia.pop if secuencia[-1] == ""
    # Y tambien nos cargamos el primero (la /)
    secuencia.shift if secuencia[0] == ""
    for i in (0..secuencia.size-1)
      #puts "-------------> Analizamos " + secuencia[i]
      if existe
        espacio = Espacio.first(:conditions => {:nombre => secuencia[i], :espacio_padre_id => objeto_id, :definicion_espacio_proyecto => nil})
        if espacio
          objeto_id = espacio.id
        else
          # Si no existe y es el ultimo, pudiera ser un documento
          if i == (secuencia.size-1) && !solo_espacios
            #documento = Documento.first(:conditions => {:espacio_id => objeto_id, :adjunto_file_name => secuencia[i]})
            documento = Documento.includes(:documento_x_espacio).where("documento_x_espacio.espacio_id" => objeto_id, :adjunto_file_name => secuencia[i]).first
            objeto_id = documento.id if documento
            existe = false unless documento
          else
            existe = false
          end
        end
      end
    end
    return existe ? objeto_id : nil 
  end

  def documento path
    objeto_id = recurso_id(path)
    Documento.find_by_id(objeto_id)
  end
  def espacio_raiz? path=file_path
    path.split('/').size == 0
  end
  def espacio path
    if espacio_raiz?(path)
      Espacio.new
    else
      objeto_id = recurso_id(path)
      Espacio.find_by_id(objeto_id)
    end
  end

	# Devuelve si existe o no existe un espacio
  def existe_espacio? path
    return espacio_raiz?(path) ? true : !objeto(path, true).nil?
  end

	# Devuelve si existe o no existe un path
  def existe_recurso? path
    return espacio_raiz?(path) ? true : !objeto(path, false).nil?
  end

        # Devuelve el path padre del recurso pedido
  def ruta_padre path=file_path
    ruta=path.split('/')
    ruta.pop
    ruta.join('/')
  end

end

#class Dav < DAV4Rack::Resource
#  #before do |resource, method_name|
#  #  resource.DAV_authenticate unless resource.user.is_a?(User)
#  #  raise Unauthorized unless resource.user.is_a?(User)
#  #end
#  #before do |resource, method_name|
#  #  resource.user.allowed?(method_name)
#  #end
#
#end
