# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# GastoXProyecto: relaciona gasto con proyecto y el porcentaje de esta relación

class GastoXProyecto < ActiveRecord::Base

  #untranslate_all

  # Metemos la validacion antes para que no se eliminen gxp de proyectos cerrados
  before_destroy :comprobar_fecha_etapas_proyecto

  # acts_as_reportable
  belongs_to :gasto
  belongs_to :proyecto
  belongs_to :subpartida
  belongs_to :marcado, :class_name => "Marcado", :foreign_key => "marcado_proyecto_id"
  belongs_to :tasa_cambio_proyecto, :class_name => "TasaCambio", :foreign_key => "tasa_cambio_id"

  #validates_presence_of :proyecto_id, :message => _("Proyecto") + " " + _("no puede estar vacío.")
  #validates_associated :proyecto, :message => _("El proyecto asociado no es correcto.")
  validates_presence_of :gasto_id, :message => _("Gasto") + " " + _("no puede estar vacío.")
  #validates_associated :gasto, :message =>  _("El gasto asociado no es correcto.")
  validates_uniqueness_of :proyecto_id, :scope => :gasto_id, :message => _("Asignación del gasto repetida.")

  validate :comprueba_importe
  # Esto valida al guardar, pero... tambien al borrar?
  validate :comprobar_fecha_etapas_proyecto

  before_save :obtiene_tasa_cambio
  # Este callback es en create para evitar cambiar cosas modificadas o no seguir validaciones de gastos
  after_create :asigna_subpartida_por_defecto

  # Comprueba que la fecha del gasto está dentro de alguna etapa del proyecto y que el proyecto no este cerrado
  def comprobar_fecha_etapas_proyecto
    # Solo comprobamos si hay proyecto (proyecto_id == nil => imputado al agente)
    if proyecto
      # Comprueba que el gasto entre dentro de las fechas de proyecto
      p_fecha_inicio = proyecto.fecha_de_inicio
      p_fecha_fin = proyecto.fecha_de_fin
      unless p_fecha_inicio and p_fecha_fin and gasto.fecha >= p_fecha_inicio and gasto.fecha <= p_fecha_fin
       errors.add("fecha", _("Proyecto '%s' no tiene definidas fechas de comienzo o de fin") % [proyecto.nombre]) unless p_fecha_inicio and p_fecha_fin
       errors.add("fecha", _("La fecha '%s' no esta dentro de las etapas del proyecto '%s'") % [gasto.fecha,proyecto.nombre]) if p_fecha_inicio and p_fecha_fin
      end

      errors.add(_("Proyecto"), _("El proyecto") + " " + _("se encuentra en estado 'cerrado'. No es posible modificar el gasto.")) if proyecto.estado_actual && proyecto.estado_actual.definicion_estado.cerrado
    end
    return errors.empty?
  end

  def comprueba_importe
    errors.add("GastoXProyecto", _("El gasto por proyecto no puede ser mayor que el total.")) if importe && importe.round(2) > gasto.importe 
    errors.add("GastoXProyecto", _("El importe vinculado al proyecto no puede estar vacío.")) unless importe
  end

  def obtiene_tasa_cambio
    # Solo actua si es nuevo o se ha cambiado el proyecto o el gasto (los cambios en moneda o fecha los hace el propio gasto)
    if self.id.nil? || self.gasto_id_changed? || self.proyecto_id_changed?
      tc = TasaCambio.tasa_cambio_para_gasto(self.gasto, self.proyecto)
      self.tasa_cambio_id = tc ? tc.id : nil
    end
  end

  # Asigna la subpartida por defecto al gasto_x_proyecto o al gasto segun quien lo haya introducido
  def asigna_subpartida_por_defecto
    # Si es un gasto originado en un proyecto
    if self.gasto.proyecto_origen_id
      # Si el gasto esta creado por este proyecto 
      if self.gasto.proyecto_origen_id == self.proyecto_id
        # Si tenemos una subpartida, probamos a vincularla
        if self.subpartida
          # Si existe la subpartida en el agente, la asignamos 
          subp_agente = Subpartida.find_by_agente_id_and_partida_id_and_nombre(self.gasto.agente_id, self.gasto.partida_id, subpartida.nombre)
          # Actualizamos sin invocar validaciones ni callbacks. No hay peligro con las validaciones de fechas en agente porque en este caso acaba de crearse el gasto
          self.gasto.update_column(:subpartida_agente_id, subp_agente.id) if subp_agente
        end  
      # Si esta creado por otro proyecto
      else
        gxp = GastoXProyecto.find_by_gasto_id_and_proyecto_id(self.gasto_id, self.gasto.proyecto_origen_id)
        # Si existe la subpartida en el proyecto origen, asignamos esa
        if gxp && gxp.subpartida
          # Si existe la subpartida en nuestro proyecto, la asignamos
          subp_nuestra = Subpartida.find_by_proyecto_id_and_partida_id_and_nombre(self.proyecto_id, self.gasto.partida_id, gxp.subpartida.nombre)
          # Actualizamos sin invocar validaciones ni callbacks
          self.update_column(:subpartida_id, subp_nuestra.id) if subp_nuestra
        end
      end
    # Si es un gasto originado en un agente
    else
      # Si existe la subpartida en nuestro proyecto, la asignamos
      if self.gasto.subpartida_agente
        subp_nuestra = Subpartida.find_by_proyecto_id_and_partida_id_and_nombre(self.proyecto_id, self.gasto.partida_id, self.gasto.subpartida_agente.nombre)
        # Actualizamos sin invocar validaciones ni callbacks
        self.update_column(:subpartida_id, subp_nuestra.id) if subp_nuestra
      end
    end
  end

  # Proyecto de la relación.
  def proyecto
    return Proyecto.find_by_id self.proyecto_id
  end

  # Tasa de Cambio asociada
  def tasa_cambio
    tc = TasaCambio.find_by_id(self.tasa_cambio_id)
    return (tc ? tc.tasa_cambio : 0.0)
  end
  # Tasa de Cambio a la divisa 
  def tasa_cambio_divisa
    tc = TasaCambio.find_by_id(self.tasa_cambio_id)
    return (tc ? tc.tasa_cambio_divisa : 0.0)
  end

  # Porcentaje en formato para la visualización
  def porcentaje_to_s
    return (100 * gasto.importe / importe ).to_i.to_s + "%"
  end

end
