# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class Periodo < ActiveRecord::Base
  belongs_to :tipo_periodo
  belongs_to :proyecto
  has_many :tarea, :dependent => :destroy

  validates_presence_of :fecha_inicio, :message => _("Fecha inicio") + " " + _("no puede estar vacío.")
  validates_presence_of :fecha_fin, :message => _("Fecha fin") + " " + _("no puede estar vacío.")

  # Esto es una chapuza que habria que eliminar (esta metido a capon el id del informe final)
  validates_uniqueness_of :tipo_periodo_id, :scope => :proyecto_id, :if => Proc.new{|periodo| periodo.tipo_periodo_id == 1}, :message => _("Informe Final repetido.")

  validate :comprobar_fechas

  def estado_color
    color = ""
    for tarea in self.tarea
      color = "verde" if tarea.estado_color == "verde" and color == ""
      color = "rosa" if tarea.estado_color == "rosa" and (color == "verde" or color == "")
      color = "rojo" if tarea.estado_color == "rojo"
    end
    return color
  end
  
  def estado_cerrado
    estado = "cerrado"
    for tarea in self.tarea
      estado = "abierto" unless tarea.estado_tarea.nil? || tarea.estado_tarea.activo
    end
    return estado == "abierto" ? false : true
  end

  def estado_abierto
    abierto = tarea.empty? 
    tarea.each { |t| abierto = true unless t.estado_tarea && !t.estado_tarea.activo }
    return abierto
  end

 private

  def comprobar_fechas
    # Comprueba que la fecha de inicio sea anterior a la de fin
    if self.fecha_fin && self.fecha_inicio && self.fecha_fin < self.fecha_inicio
      errors.add("fecha_fin", _("Fecha fin tiene que ser mayor que fecha inicio"))
    elsif self.fecha_inicio && self.fecha_fin
      errores = false
      # Comprueba que no se solapen fechas (una fecha de inicio de un periodo anterior a la de fin de otro)
      (proyecto.periodo.where(:tipo_periodo_id => self.tipo_periodo_id) - [self]).each do |p|
        errores = true unless (self.fecha_inicio < p.fecha_inicio && self.fecha_fin < p.fecha_inicio) || (self.fecha_inicio > p.fecha_fin && self.fecha_fin > p.fecha_fin)
      end
      errors.add("fecha_inicio", _("El periodo no puede solaparse con otro")) if errores
    end
    return errors.nil?
  end

end
