# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de la matriz. Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para los objetivos especificos, resultados, actividades...

class IndicadorController < ApplicationController

  before_filter :verificar_estado_formulacion_ajax, :only => [	:crear_modificar_variable, :eliminar_variable ]
  before_filter :verificar_estado_ejecucion_ajax, :only => [	:crear_modificar_valor_indicador, :eliminar_valor_indicador,
								:crear_modificar_valor_variable, :eliminar_valor_variable ]

  def verificar_estado_formulacion_ajax
    unless @permitir_formulacion
      render :update do |page|
        mensaje = _("El proyecto se encuentra 'sin estado'.") + " " + _("Es necesario definirle un estado al proyecto.")    if @proyecto.estado_actual.nil?
        mensaje = _("El proyecto se encuentra en estado '%{estado}'.")%{:estado => @proyecto.estado_actual.definicion_estado.nombre} + " " + _("En este estado no se pueden modificar los indicadores.") unless @proyecto.estado_actual.nil?
        page.mensaje_informacion params[:update], mensaje, :tipo_mensaje => "mensajefallo"
      end
    end
  end

  def verificar_estado_ejecucion_ajax
    unless @permitir_ejecucion
      render :update do |page|
        mensaje = _("El proyecto se encuentra 'sin estado'.") + " " + _("Es necesario definirle un estado al proyecto.")    if @proyecto.estado_actual.nil?
        mensaje = _("El proyecto se encuentra en estado '%{estado}'.")%{:estado => @proyecto.estado_actual.definicion_estado.nombre} + " " + _("En este estado no se puede hacer seguimiento de los indicadores.") unless @proyecto.estado_actual.nil?
        page.mensaje_informacion params[:update], mensaje, :tipo_mensaje => "mensajefallo"
      end
    end
  end


  #--
  # LISTADO DE INDICADORES
  #--

  def index
    redirect_to :action => "listado"
  end

        # en proyectos: listado de variables de indicadores
  def listado
    # Segun de donde se venga, se recoge el indicador elegido (puede venir desde el filtrado o desde el listado de indicadores)
    indicador_id = params[:indicador_id] if params[:indicador_id] && !params[:indicador_id].nil?
    indicador_id = params[:filtro][:indicador] if params[:filtro] && params[:filtro][:indicador] != "todos"

    condiciones = ["objetivo_especifico.proyecto_id=? OR resultado.proyecto_id=?",@proyecto.id,@proyecto.id] unless indicador_id
    condiciones = ["id=?",indicador_id] if indicador_id
    @indicadores = Indicador.find(:all,:order => "indicador.codigo", :include=>["objetivo_especifico","resultado"],:conditions => condiciones)
  end


  #--
  # MEDICIONES DEL INDICADOR
  #++

  def valores_indicador
    @valores = ValorIntermedioXIndicador.all(:order => "fecha DESC", :conditions => {:indicador_id => params[:indicador_id]})
    render(:update) { |page| page.replace_html params[:update], :partial => "valores_indicador", :locals => {:update_listado => params[:update]} }
  end

  def nuevo_editar_valor_indicador
    @indicador = Indicador.find_by_id(params[:indicador_id])
    @valor = ValorIntermedioXIndicador.find_by_id(params[:id]) || ValorIntermedioXIndicador.new()
    render(:update) { |page| page.formulario :partial => "formulario_valor_indicador", :update => params[:update] }
  end

  def crear_modificar_valor_indicador
    @indicador = Indicador.find_by_id(params[:indicador_id])
    @valor = @indicador.valor_intermedio_x_indicador.find_by_id(params[:id]) || ValorIntermedioXIndicador.new(:indicador_id => params[:indicador_id])
    # Cualquier modificacion o creacion es del usuario que modifica
    params[:valor][:usuario_id] = @usuario_identificado.id
    # Actualizamos
    @valor.update_attributes params[:valor]
    # Si no hay errores al guardar la medicion...
    if @valor.errors.empty?
      @valores = ValorIntermedioXIndicador.all(:order => "fecha DESC", :conditions => {:indicador_id => params[:indicador_id]})
      render(:update) do |page|
        # Primero actualizamos la linea del indicador
        page.modificar :update => params[:update_objeto], :partial => "indicador", :mensaje => { :errors => @valor.errors }, :locals => {:fila => params[:update_objeto]}
        # ... y luego actualizamos el listado de valores
        page.replace_html params[:update_listado], :partial => "valores_indicador", :locals => {:update_listado => params[:update_listado]}
      end
    # ... si hay errores al guardar la medicion, volvemos al formulario
    else
      render(:update){ |page|  page.recargar_formulario :partial => "formulario_valor_indicador", :mensaje => { :errors => @valor.errors } }
    end
  end

  def eliminar_valor_indicador
    @indicador = Indicador.find_by_id(params[:indicador_id])
    @objeto = ValorIntermedioXIndicador.find_by_id(params[:id])
    @objeto.destroy
    @valores = ValorIntermedioXIndicador.all(:order => "fecha DESC", :conditions => {:indicador_id => params[:indicador_id]})
    render(:update) do |page|
      # Primero actualizamos la linea del indicador
      page.actualizar :update => params[:update_objeto], :partial => "indicador", :mensaje => { :errors => @objeto.errors, :eliminar => true }, :locals => {:fila => params[:update_objeto]}
      # ... y luego actualizamos el listado de valore
      page.replace_html params[:update], :partial => "valores_indicador", :locals => {:update_listado => params[:update]}
    end 
  end

  #--
  # VARIABLES DE INDICADORES
  #++

	# en proyectos: prepara el formulario de edicion o creacion de una variable de indicador
  def editar_variable
    @variable = params[:id] ? VariableIndicador.find(params[:id]) : VariableIndicador.new(:indicador_id => params[:indicador_id])
    if @variable.id
      @valor_base = @variable.valor_base
      @valor_objetivo = @variable.valor_objetivo
    end 
    render(:update) { |page| page.formulario :partial => "formulario_variable", :update => params[:update] }
  end

	# en proyectos modifica o crea una variable de indicador
  def crear_modificar_variable
    @variable = params[:id] ? VariableIndicador.find(params[:id]) : VariableIndicador.new(:indicador_id => params[:indicador_id])
    @variable.update_attributes params[:variable]
    @indicador = @variable.indicador
    if @variable.errors.empty?
      valor_base = @variable.valor_base || ValorVariableIndicador.new()
      valor_base.update_attributes params[:valor_base]
      @variable.valor_base = valor_base

      valor_objetivo = @variable.valor_objetivo || ValorVariableIndicador.new()
      valor_objetivo.update_attributes params[:valor_objetivo]
      @variable.valor_objetivo = valor_objetivo

      @variable.save
    end

    render(:update){ |page| page.modificar( :update_listado => params[:update_listado], :partial => "variables", :mensaje => { :errors => @variable.errors }) } if @variable.id
    render(:update){ |page| page.recargar_formulario :partial => "formulario_variable", :mensaje => { :errors => @variable.errors } } unless @variable.id
  end

	# en proyectos: elimina la variable de un indicador
  def eliminar_variable
    @variable = VariableIndicador.find(params[:id])
    @indicador = @variable.indicador
    @variable.destroy
    render(:update) { |page|  page.eliminar(:update => params[:update], :mensaje => { :errors => @variable.errors, :eliminar => true }) }
  end


  #--
  # VALORES DE VARIABLES DE INDICADOR
  #++

	# en proyectos, presenta un sublistado de valores intermedios de variables
  def valores_variable
    variable = VariableIndicador.find(params[:variable_indicador_id])
    @valores = variable.valor_medido.find(:all,:order => "fecha")
    render(:update) { |page| page.replace_html params[:update], :partial => "valores_variable", :locals => {:update_listado => params[:update]} }
  end

  def nuevo_editar_valor_variable
    @valor_variable = params[:id] ? ValorVariableIndicador.find(params[:id]) : ValorVariableIndicador.new
    render(:update) { |page| page.formulario :partial => "formulario_valor_variable", :update => params[:update] }
    #render :partial => "formulario_valor_variable"
  end

  def crear_modificar_valor_variable
    @valor_variable = params[:id] ? ValorVariableIndicador.find(params[:id]) : ValorVariableIndicador.new(:variable_indicador_id => params[:variable_indicador_id])
    @valor_variable.update_attributes params[:valor_variable]
    @valores = VariableIndicador.find(params[:variable_indicador_id]).valor_medido.find(:all, :order => "fecha")
    if @valor_variable.id
      render(:update){ |page|  page.modificar(:update_listado => params[:update_listado], :partial => "valores_variable", :mensaje => { :errors => @valor_variable.errors}, :tipo_update => "sublistado") }
    else
      render(:update){ |page|  page.recargar_formulario :partial => "formulario_valor_variable", :mensaje => { :errors => @valor_variable.errors } }
    end
  end

  def eliminar_valor_variable
    @objeto = ValorVariableIndicador.find(params[:id])
    @objeto.destroy
    @valores = VariableIndicador.find(params[:variable_indicador_id]).valor_medido.find(:all, :order => "fecha")
    render(:update) { |page| page.replace_html params[:update], :partial => "valores_variable", :locals => {:update_listado => params[:update]} }
  end

end
